@@ -481,6 +481,26 @@ class database:
         with SCPClient(ssh.get_transport()) as scp:
            scp.put(localfile, remotedir)
         
+    def remove_file(self,index):
+        """This routine removes a file from the data base locally.
+           The file is idenfied by the index within the table.
+           
+        """
+
+        # check whether file is downloaded and get hashname
+        hashname=self.pddf.iloc[2]['hashname']
+        # try to remove file 
+        # first copy to working directory
+        try:
+          os.remove(self.workdir+hashname)
+        except:
+          pass
+        # drop row from table
+        self.pddf=self.pddf.drop(self.pddf.index[[index]])
+        self.pddf.reset_index(drop=True, inplace=True)
+        # and save table to disc 
+        savedf=self.pddf.copy(deep=True)
+        savedf.to_hdf(self.workdir+self.pddf_hdf, key='densityfiles', mode='w')
         
     def merge(self,old):
        """Merges two databases for density files
@@ -524,8 +544,9 @@ class database:
 class densfile2b:
     """This class defines an object that allows to read a 2-body density file into numpy arrays
        and to compress a file."""
-    def  __init__(self,filen_in):
+    def  __init__(self,filen_in,printlevel=0):
         """This opens a 2b density file for reading and reads all info an prints simple statistics.
+           optional parameter printlevel controls amout of printout: 0 = none, 1 = limited, 2 = extensive 
         
            After using this method, the kinematics and data are properties of the objects
               - maxrhoindex: number of non-zero channel combinations 
@@ -561,12 +582,14 @@ class densfile2b:
         self.num2Nchan=filein['num2Nchan'][()]
         self.qn2Nchan=filein['qn2Nchan'][()]
         self.rhoindx=filein['rhoindx'][()]
-
-        print('Number of channels and couplings:   {0:d}  {1:d}\n'.format(self.num2Nchan,self.maxrhoindex))
-        print('Channels:\n')
-        print("{0:>5s} {1:>3s} {2:>3s} {3:>3s} {4:>3s} {5:>3s} {6:>3s}  {7:>3s}".format(
-        "alpha","l","s","j","t","mt","mj","2.MJ"))
-        for alpha in range(self.num2Nchan):
+        
+        if printlevel>0:
+          print('Number of channels and couplings:   {0:d}  {1:d}\n'.format(self.num2Nchan,self.maxrhoindex))
+        if printlevel>1:  
+          print('Channels:\n')
+          print("{0:>5s} {1:>3s} {2:>3s} {3:>3s} {4:>3s} {5:>3s} {6:>3s}  {7:>3s}".format(
+          "alpha","l","s","j","t","mt","mj","2.MJ"))
+          for alpha in range(self.num2Nchan):
             print("{0:5d} {1:3d} {2:3d} {3:3d} {4:3d} {5:3d} {6:3d}  {7:3d}".format(alpha,
                    self.qn2Nchan[alpha,0],self.qn2Nchan[alpha,1],self.qn2Nchan[alpha,2],
                    self.qn2Nchan[alpha,3],self.qn2Nchan[alpha,4],self.qn2Nchan[alpha,5],
@@ -587,12 +610,13 @@ class densfile2b:
         self.p12p=filein['p12p'][()]
         self.p12w=filein['p12w'][()]
 
-
-        print("MESHTYPE: {0:s}".format(self.meshtype))
-        print("NP12:     {0:d} {1:d}  {2:d} ".format(self.p12n1,self.p12n2,self.p12ntot))
-        print("P12:      {0:15.6e}  {1:15.6e}  {2:15.6e} \n ".format(self.p12a,self.p12b,self.p12c))
-        print("Grid points: \n")
-        for ip in range(self.p12ntot):
+        if printlevel>0:
+          print("MESHTYPE: {0:s}".format(self.meshtype))
+          print("NP12:     {0:d} {1:d}  {2:d} ".format(self.p12n1,self.p12n2,self.p12ntot))
+          print("P12:      {0:15.6e}  {1:15.6e}  {2:15.6e} \n ".format(self.p12a,self.p12b,self.p12c))
+        if printlevel>1:
+          print("Grid points: \n")
+          for ip in range(self.p12ntot):
             print("P12MESH:   {0:15.6e} {1:15.6e}".format(self.p12p[ip],self.p12w[ip]))
 
 
@@ -607,9 +631,10 @@ class densfile2b:
               self.qval=group["qval"][()]
               self.thetaqval=group["thetaqval"][()]
               self.thetaval=group["thetaval"][()]
-              print("OMEGA   = {0:15.6e}  {1:15.6e}".format(om*hbarc,self.omval*hbarc))
-              print("THETA   = {0:15.6e}  {1:15.6e}".format(th*180.0/np.pi,self.thetaval*180.0/np.pi))
-              print("q,theta = {0:15.6e}  {1:15.6e}".format(self.qval*hbarc,self.thetaqval*180.0/np.pi))
+              if printlevel>0:
+                print("OMEGA   = {0:15.6e}  {1:15.6e}".format(om*hbarc,self.omval*hbarc))
+                print("THETA   = {0:15.6e}  {1:15.6e}".format(th*180.0/np.pi,self.thetaval*180.0/np.pi))
+                print("q,theta = {0:15.6e}  {1:15.6e}".format(self.qval*hbarc,self.thetaqval*180.0/np.pi))
     
         # integrate to get normalization or ff 
         
@@ -622,17 +647,19 @@ class densfile2b:
             else:    
                 ffsum2+=np.einsum("ii,i,i->",self.rhoarray[rindx-1,:,:],self.p12p**2,self.p12w)
 
-        print("     {0:>15s} {1:>15s}    {2:>15s}  {3:>15s}".format("qval[fm-1]","qval[MeV]","ffsum1","ffsum2"))
-        print("FF:  {0:15.6e} {1:15.6e}    {2:15.6e}  {3:15.6e}".format(self.qval,self.qval*hbarc,ffsum1,ffsum2))
+        if printlevel>0:
+          print("     {0:>15s} {1:>15s}    {2:>15s}  {3:>15s}".format("qval[fm-1]","qval[MeV]","ffsum1","ffsum2"))
+          print("FF:  {0:15.6e} {1:15.6e}    {2:15.6e}  {3:15.6e}".format(self.qval,self.qval*hbarc,ffsum1,ffsum2))
 
 
         filein.close()
-        print("File size read:  {0:d}".format(os.path.getsize(filen_in)))
+        if printlevel>0:
+          print("File size read:  {0:d}".format(os.path.getsize(filen_in)))
         
 
-    def compress(self,filen_out,tolerance):
+    def compress(self,filen_out,tolerance,printlevel=0):
         """This method write the current data in compressed form to the new file file_out using
-           ZFP compression with accuracy tolerance."""
+           ZFP compression with accuracy tolerance. Optional parameter printlevel controls printout."""
     
         fileout=h5py.File(filen_out, 'w')
         
@@ -692,12 +719,13 @@ class densfile2b:
         dsetout=groupout.create_dataset("thetaval",self.thetaval.shape,dtype="f8") 
         dsetout[()]=self.thetaval        
         fileout.close()
-        print("File size written:  {0:d}".format(os.path.getsize(filen_out)))
+        if printlevel>0:
+          print("File size written:  {0:d}".format(os.path.getsize(filen_out)))
 
     
 class densfile1b:
     """This class defines an object that allows to read a 1-body density file into numpy arrays"""
-    def  __init__(self,filen_in):
+    def  __init__(self,filen_in,printlevel=0):
         """This opens a 1b density file for reading and reads all info an prints simple statistics.
         
            After using this method, the kinematics and data are properties of the objects
@@ -732,9 +760,12 @@ class densfile1b:
         self.rho1bindx=np.array(rho1bindxlist).reshape(self.maxrho1bindex,8)
 
         # print bookkeeping
-        print("{0:>5s}  {1:>4s}  {2:>4s}  {3:>4s}  {4:>4s}  {5:>4s}  {6:>4s}  {7:>4s}  {8:>4s}".format(
-               "chan","2.ms","2.mt","2.M","2.msp","2.mtp","2.Mp","k","K"))
-        for i in range(self.maxrho1bindex):
+        if printlevel>0:
+          print("maxrho1bindex = {0:d}".format(self.maxrho1bindex))
+        if printlevel>1:
+          print("{0:>5s}  {1:>4s}  {2:>4s}  {3:>4s}  {4:>4s}  {5:>4s}  {6:>4s}  {7:>4s}  {8:>4s}".format(
+                 "chan","2.ms","2.mt","2.M","2.msp","2.mtp","2.Mp","k","K"))
+          for i in range(self.maxrho1bindex):
             print("{0:5d}  {1:4d}  {2:4d}  {3:4d}  {4:4d}  {5:4d}  {6:4d}  {7:4d}  {8:>4d}".format(i,
                    self.rho1bindx[i,0],self.rho1bindx[i,1],self.rho1bindx[i,2],
                    self.rho1bindx[i,3],self.rho1bindx[i,4],
@@ -793,9 +824,10 @@ class densfile1b:
             if mt==-1:  #  neutron 
                 sumn=sumn+self.rho1b[rindx]
             sum=sum+self.rho1b[rindx]
-        print("FF1B:   {0:15.6e} {1:15.6e} {2:15.6e} {3:15.6e}".format(self.qval,sumn,sump,sum) )  
-        print("omega  =   {0:15.6e} ".format(self.omval*hbarc) )  
-        print("q      =   {0:15.6e} ".format(self.qval*hbarc) )  
-        print("theta  =   {0:15.6e} ".format(self.thetaval*180.0/np.pi) )  
-        print("thetaq =   {0:15.6e} ".format(self.thetaqval*180.0/np.pi) )  
+        if printlevel>0:
+          print("FF1B:   {0:15.6e} {1:15.6e} {2:15.6e} {3:15.6e}".format(self.qval,sumn,sump,sum) )  
+          print("omega  =   {0:15.6e} ".format(self.omval*hbarc) )  
+          print("q      =   {0:15.6e} ".format(self.qval*hbarc) )  
+          print("theta  =   {0:15.6e} ".format(self.thetaval*180.0/np.pi) )  
+          print("thetaq =   {0:15.6e} ".format(self.thetaqval*180.0/np.pi) )  
  
